/* string_transmission_protocoll.h
 *
 * Copyright (c) 2010 Stefan Beller.  Use may be in
 * whole or in part in accordance to the General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


#ifndef __STRING_TRANSPORT_PROTOCOLL_H__
	#define __STRING_TRANSPORT_PROTOCOLL_H__
	
	#include <stdint.h>		/* for uint32_t */
	#include <errno.h>		/* for errno */
	#include <stdio.h>      /* for perror() and printf() */
	#include <stdlib.h>     /* for atoi() and exit() */
	#include <string.h>     /* for memset() */
	#include <sys/socket.h> /* for socket(), connect(), send(), and recv() */
	#include <unistd.h>     /* for close() */
	#include <netdb.h> 		/* gethostbyname() */
	//#include <sys/ioctl.h>
	#include <pthread.h>	/* for pthread_create() in stp_accept_connections*/
	
	typedef struct{
		int socket; 			//which socket do i use?
		char* waitingline; 		//pointer to a buffer where the message will be buffered, 
								//this will be allocated when recv_line is called.
		int offset; 			//offset to waitingline where the next received byte shall be put
		int maxSize;			//stores the size of waitingline
		pthread_mutex_t smutex;	//this mutex will be set if you send, so send will be threadsafe
		pthread_mutex_t rmutex;	//this mutex will be set if you recv, so recv will be threadsafe
	}stp_connectionstruct;

	/*users should not see pointers, so hide them in stp_connection*/
	typedef stp_connectionstruct* stp_connection;

	typedef struct {
		int port;		// at which port should be listened
		int running;	// boolean: is the accepter thread even running?
		void (*callback)(stp_connection new_connection);
		pthread_t* accepting_thread;
		pthread_attr_t* accepting_thread_attr; 
	}stp_acceptingstruct;
	
	/*users should not see pointers, so hide them in stp_accepter */
	typedef stp_acceptingstruct* stp_accepter;	

	
	/*in mikro-seconds; defines how long to sleep, if nothing has happend in stp_accept_connections*/
	#define STP_ACCEPT_SLEEPTIME 	1000  
	
	/*in mikro-seconds; defines how long to sleep, if nothing has arrived in stp_recv*/
	#define STP_RECV_SLEEPTIME		100
	
	/*
	 * defines the datatype in which the length of transmissions will be stored,
	 * the maximum number of STP_MAXLENGTH_DATATYPE determines how many 
	 * bytes can be transmitted via a single transmission
	 */
	#define STP_MAXLENGTH_DATATYPE uint16_t
	
	/* 
	 * This datatype is used to store the length of a transmission. 
	 * To get it transmitted, it must be accessible as a bytes buffer as well. 
	 */
	typedef union{
		STP_MAXLENGTH_DATATYPE length_int;
		char length_str[sizeof(STP_MAXLENGTH_DATATYPE)];
	}stp_length_t;
		
	/*
	 * This function will connect to the given Host at Port.
	 * maxStringSize specifies the maximal Size of Strings which can be transmitted by the Connections.
	 * if connection fails, NULL will be returned.
	 */
	stp_connection stp_connect(char* host, int port);
	
	/*
	 * This function listens for connections at Port <port>. 
	 * maxStringSize specifies the maximal Size of Strings which can be transmitted by the Connections.
	 * callback is a function which is called when there is a new incoming connection.
	 * when setting up the socket and listening for clientes fails, callback(NULL) will be called.
	 * 
	 */
	stp_accepter stp_accept_connections(int port, void callback(stp_connection new_connection));
	
	void stp_stop_accepting(stp_accepter accepter);
	
	/* This function blocks until a complete String is received.
	 * The received String will be returned.
	 * if the connection fails, NULL will be returned.
	 */
	char* stp_recv(stp_connection stp );
	
	/* This function blocks until complete String is transmitted
	 * errno will be returned , 0 normally
	 */ 
	int stp_send(stp_connection stp, char* line);
	
	/* shutdown a connection. 
	 * it will send zeros to the other socket, so that it notices this shutdown
	 * frees all memory of the connection
	 */
	void stp_shutdown(stp_connection stp );
	
#endif
